package ru.risdeveau.geotracker

import android.Manifest.permission.ACCESS_FINE_LOCATION
import android.Manifest.permission.POST_NOTIFICATIONS
import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.app.Service
import android.content.Intent
import android.os.Build
import android.os.IBinder
import android.util.Log
import androidx.core.app.NotificationCompat
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.Job
import kotlinx.coroutines.launch


class LocationForegroundService : Service() {

    private lateinit var locationTracker: LocationTracker
    private val serviceScope = CoroutineScope(Dispatchers.Main + Job())

    override fun onCreate() {
        Log.d("Service", "onCreate")
        super.onCreate()
        locationTracker = LocationTracker(this) { location ->
            serviceScope.launch {
                sendGeo(
                    data = GeoData(
                        lt = location.latitude,
                        ln = location.longitude,
                        nick = SettingsPreferences.username
                    )
                )
            }
        }
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        Log.d("Service", "onStartCommand")

        if (
            !(hasPermission(ACCESS_FINE_LOCATION)
            && (
                (Build.VERSION.SDK_INT < 33)
                || hasPermission(POST_NOTIFICATIONS)
            ))
            || intent?.action == ACTION_STOP_SERVICE
        ) {
            stopSelf()
            return START_NOT_STICKY
        }

        createNotificationChannel()
        val notification = createNotification()
        startForeground(1, notification)

        locationTracker.startTracking(SettingsPreferences.interval * 1000L)
        return START_STICKY
    }

    override fun onDestroy() {
        Log.d("Service", "Destroyed")
        locationTracker.stopTracking()
        super.onDestroy()
    }

    private fun createNotificationChannel() {
        Log.d("Service", "createNotificationChannel")
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val channel = NotificationChannel(
                "location_channel",
                "Отправка Местоположения",
                NotificationManager.IMPORTANCE_HIGH
            )
            val manager = getSystemService(NOTIFICATION_SERVICE) as NotificationManager
            manager.createNotificationChannel(channel)
        }
    }

    private fun createNotification(): Notification {
        val stopIntent = Intent(this, LocationForegroundService::class.java).apply {
            action = ACTION_STOP_SERVICE
        }

        val stopPendingIntent = PendingIntent.getService(
            this,
            0,
            stopIntent,
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S)
                PendingIntent.FLAG_UPDATE_CURRENT or PendingIntent.FLAG_MUTABLE
            else
                PendingIntent.FLAG_UPDATE_CURRENT
        )

        return NotificationCompat.Builder(this, "location_channel")
            .setContentTitle("Отслеживание местоположения")
            .setContentText("Обновление каждые ${SettingsPreferences.interval} секунд")
            .setSmallIcon(R.drawable.share_location)
            .addAction(
                R.drawable.cancel,
                "Остановить",
                stopPendingIntent
            )
            .setOngoing(true)
            .build()
    }

    override fun onBind(intent: Intent?): IBinder? = null

    companion object {
        const val ACTION_STOP_SERVICE = "ru.risdeveau.geotracker.STOP_SERVICE"
    }
}
